@@ -251,8 +251,21 @@ start_window_manager(struct login_info *login_info,
         }
     }
 
-    if (s->shell[0] != '\0')
+    if (g_cfg->sec.allow_alternate_shell &&
+            g_cfg->sec.pass_shell_as_env != NULL &&
+            g_cfg->sec.pass_shell_as_env[0] != '0')
     {
+        // Pass the shell in to the standard startwm scripts
+        // in an environment variable
+        LOG(LOG_LEVEL_INFO,
+            "Setting variable '%s' to the specified shell of '%s'",
+            g_cfg->sec.pass_shell_as_env,
+            s->shell);
+        g_setenv_log(g_cfg->sec.pass_shell_as_env, s->shell, 1);
+    }
+    else if (s->shell[0] != '\0')
+    {
+        // Try to execute the shell directly (if permitted)
         if (g_cfg->sec.allow_alternate_shell)
         {
             if (g_strchr(s->shell, ' ') != 0 || g_strchr(s->shell, '\t') != 0)
@@ -611,6 +624,19 @@ session_start_wrapped(struct login_info *login_info,
     int window_manager_pid;
     enum scp_screate_status status = E_SCP_SCREATE_GENERAL_ERROR;
 
+    /* Set the secondary groups before starting the session to prevent
+     * problems on PAM-based systems (see Linux pam_setcred(3)).
+     * If we have *BSD setusercontext() this is not done here */
+#ifndef HAVE_SETUSERCONTEXT
+    if (g_initgroups(login_info->username) != 0)
+    {
+        LOG(LOG_LEVEL_ERROR,
+            "Failed to initialise secondary groups for %s: %s",
+            login_info->username, g_get_strerror());
+        return E_SCP_SCREATE_GENERAL_ERROR;
+    }
+#endif
+
     if (auth_start_session(login_info->auth_info, s->display) != 0)
     {
         // Errors are logged by the auth module, as they are
@@ -637,19 +663,6 @@ session_start_wrapped(struct login_info *login_info,
     }
 #endif
 
-    /* Set the secondary groups before starting the session to prevent
-     * problems on PAM-based systems (see Linux pam_setcred(3)).
-     * If we have *BSD setusercontext() this is not done here */
-#ifndef HAVE_SETUSERCONTEXT
-    if (g_initgroups(login_info->username) != 0)
-    {
-        LOG(LOG_LEVEL_ERROR,
-            "Failed to initialise secondary groups for %s: %s",
-            login_info->username, g_get_strerror());
-        return E_SCP_SCREATE_GENERAL_ERROR;
-    }
-#endif
-
     /* start the X server in a new process group.
      *
      * We group the X server, window manager and chansrv in a single
