@@ -426,14 +426,30 @@ check_drop_privileges(struct xrdp_startup_params *startup_params)
     int rv = 1;
     const char *user = startup_params->runtime_user;
     const char *group = startup_params->runtime_group;
+    int entry_uid = g_getuid();
+    int entry_gid = g_getgid();
 
-    if (user[0] == '\0' && group[0] == '\0')
+    // Regard a user as privileged if either the UID or GID is zero
+    int privileged_user = (entry_uid == 0 || entry_gid == 0);
+    if (!privileged_user)
     {
-        // Allow this for now
-        LOG(LOG_LEVEL_ALWAYS,
-            "You are running xrdp as root. This is not safe.");
+        // xrdp daemon UID:GID is already set by whatever started it. We'll
+        // log this as it may be unexpected and continue. We don't have
+        // any privilege to override this.
+        LOG(LOG_LEVEL_INFO,
+            "xrdp is already running as an unprivileged user uid=%d gid=%d",
+            entry_uid, entry_gid);
         rv = 0;
     }
+    else if (user[0] == '\0' && group[0] == '\0')
+    {
+        // We're running as a privileged user, and this isn't
+        // overridden in our own config
+        LOG(LOG_LEVEL_ALWAYS,
+            "You are running xrdp as uid=%d gid=%d. This is not recommended.",
+            entry_uid, entry_gid);
+        rv = 0; // Allowed for now for old configs
+    }
     else if (user[0] == '\0' || group[0] == '\0')
     {
         LOG(LOG_LEVEL_ERROR,
@@ -536,6 +552,7 @@ main(int argc, char **argv)
     enum logReturns error;
     struct xrdp_startup_params startup_params = {0};
     int pid;
+    int daemon_pid = 0;
     int daemon;
     char text[256];
     const char *pid_file = XRDP_PID_PATH "/xrdp.pid";
@@ -707,6 +724,7 @@ main(int argc, char **argv)
             g_exit(0);
         }
 
+        daemon_pid = g_getpid();
         g_sleep(1000);
         /* write our pid to file */
         g_sprintf(text, "%d", g_getpid());
@@ -795,7 +813,7 @@ main(int argc, char **argv)
     g_delete_wait_obj(g_get_sync_event());
     g_set_sync_event(0);
 
-    if (daemon)
+    if (daemon && (daemon_pid == g_getpid()))
     {
         /* Try to delete the PID file, although if we've dropped
          * privileges this won't be successful */
