@@ -303,7 +303,8 @@ xrdp_wm_ok_clicked(struct xrdp_bitmap *wnd)
 */
 static int
 xrdp_wm_parse_domain_information(char *originalDomainInfo, int comboMax,
-                                 int decode, char *resultBuffer)
+                                 int decode,
+                                 char *resultBuffer, unsigned int resultSize)
 {
     int ret;
     int pos;
@@ -313,8 +314,7 @@ xrdp_wm_parse_domain_information(char *originalDomainInfo, int comboMax,
     /* If the first char in the domain name is '_' we use the domain
        name as IP*/
     ret = 0; /* default return value */
-    /* resultBuffer assumed to be 256 chars */
-    g_memset(resultBuffer, 0, 256);
+    g_memset(resultBuffer, 0, resultSize);
     if (originalDomainInfo[0] == '_')
     {
         /* we try to locate a number indicating what combobox index the user
@@ -324,7 +324,7 @@ xrdp_wm_parse_domain_information(char *originalDomainInfo, int comboMax,
          * Invalid chars are ignored in microsoft client therefore we use '_'
          * again. this sec '__' contains the split for index.*/
         pos = g_pos(&originalDomainInfo[1], "__");
-        if (pos > 0)
+        if (pos > 0 && (unsigned int)pos < resultSize)
         {
             /* an index is found we try to use it */
             LOG(LOG_LEVEL_DEBUG, "domain contains index char __");
@@ -351,7 +351,7 @@ xrdp_wm_parse_domain_information(char *originalDomainInfo, int comboMax,
         else
         {
             LOG(LOG_LEVEL_DEBUG, "domain does not contain _");
-            g_strncpy(resultBuffer, &originalDomainInfo[1], 255);
+            g_strncpy(resultBuffer, &originalDomainInfo[1], resultSize - 1);
         }
     }
     return ret;
@@ -484,7 +484,8 @@ xrdp_wm_show_edits(struct xrdp_wm *self, struct xrdp_bitmap *combo)
                     {
                         xrdp_wm_parse_domain_information(
                             self->session->client_info->domain,
-                            combo->data_list->count, 0, resultIP);
+                            combo->data_list->count, 0,
+                            resultIP, sizeof(resultIP));
                         g_strncpy(b->caption1, resultIP, 255);
                         b->edit_pos = utf8_char_count(b->caption1);
                     }
@@ -999,7 +1000,8 @@ xrdp_login_wnd_create(struct xrdp_wm *self)
     combo->item_index = xrdp_wm_parse_domain_information(
                             self->session->client_info->domain,
                             combo->data_list->count, 1,
-                            resultIP /* just a dummy place holder, we ignore */ );
+                            resultIP,/* just a dummy place holder, we ignore */
+                            sizeof(resultIP));
     xrdp_wm_show_edits(self, combo);
 
     return 0;
