@@ -37,6 +37,7 @@
 #include "log.h"
 #include "xrdp_sockets.h"
 #include "string_calls.h"
+#include "channel_defs.h"
 #include "xrdpapi.h"
 
 struct wts_obj
@@ -45,6 +46,17 @@ struct wts_obj
     int display_num;
 };
 
+/**
+ * Data we store for each server
+ */
+struct wts_server
+{
+    struct wts_obj *info_obj; // Object to get session notifications
+    struct xrdp_chan_session_state session_state; // session state
+};
+
+static struct wts_server wts_current_server;
+
 /* helper functions used by WTSxxx API - do not invoke directly */
 static int
 can_send(int sck, int millis, int restart);
@@ -54,61 +66,69 @@ static int
 mysend(int sck, const void *adata, int bytes);
 static int
 myrecv(int sck, void *adata, int bytes);
+static int
+mypeek(int sck, void *adata, int bytes);
 
-/*
- * Opens a handle to the server end of a specified virtual channel - this
- * call is deprecated - use WTSVirtualChannelOpenEx() instead
- *
- * @param  hServer
- * @param  SessionId     - current session ID; *must* be WTS_CURRENT_SERVER_HANDLE
- * @param  pVirtualName  - virtual channel name when using SVC
- *                       - name of endpoint listener when using DVC
- *
- * @return a valid pointer on success, NULL on error
- ******************************************************************************/
-void *
-WTSVirtualChannelOpen(void *hServer, unsigned int SessionId,
-                      const char *pVirtualName)
+static void
+free_wts(struct wts_obj *wts)
 {
-    if (hServer != WTS_CURRENT_SERVER_HANDLE)
+    if (wts != NULL)
     {
-        return 0;
+        if (wts->fd >= 0)
+        {
+            close(wts->fd);
+        }
+        free(wts);
     }
-
-    return WTSVirtualChannelOpenEx(SessionId, pVirtualName, 0);
 }
 
 /*
  * Opens a handle to the server end of a specified virtual channel
  *
- * @param  SessionId     - current session ID; *must* be WTS_CURRENT_SERVER_HANDLE
+ * @param  SessionId     - current session ID; *must* be WTS_CURRENT_SESSION
  * @param  pVirtualName  - virtual channel name when using SVC
  *                       - name of endpoint listener when using DVC
  * @param  flags         - type of channel and channel priority if DVC
+ * @param  private_chan  - If != 0, this is a private channel defined
+ *                          in channel_defs.h
+ * @param[out] errcode   - Indication for the user of a possible
+ *                         error. Cannot be defaulted. Is only set on error.
  *
  * @return a valid pointer on success, NULL on error
  ******************************************************************************/
-void *
-WTSVirtualChannelOpenEx(unsigned int SessionId, const char *pVirtualName,
-                        unsigned int flags)
+static void *
+VirtualChannelOpen(unsigned int SessionId, const char *pVirtualName,
+                   unsigned int flags,
+                   unsigned int private_chan,
+                   enum wts_errcode *errcode)
 {
     struct wts_obj *wts;
     int bytes;
     unsigned long long1;
     struct sockaddr_un s;
-    char *connect_data;
-    int chan_name_bytes;
+    // Pad the connect data out to a larger size to allow for
+    // changes to struct xrdp_chan_connect
+    union
+    {
+        char pad[XRDPAPI_CONNECT_PDU_LEN];
+        struct xrdp_chan_connect connect_data;
+    } cd = {0};
+
+    uint32_t connect_result;
     int lerrno;
 
     if (SessionId != WTS_CURRENT_SESSION)
     {
         LOG(LOG_LEVEL_ERROR, "WTSVirtualChannelOpenEx: bad SessionId");
+        *errcode = WTS_E_BAD_SESSION_ID;
         return 0;
     }
+
     wts = (struct wts_obj *) calloc(1, sizeof(struct wts_obj));
     if (wts == NULL)
     {
         LOG(LOG_LEVEL_ERROR, "WTSVirtualChannelOpenEx: calloc failed");
+        *errcode = WTS_E_RESOURCE_ERROR;
         return 0;
     }
     wts->fd = -1;
@@ -116,7 +136,8 @@ WTSVirtualChannelOpenEx(unsigned int SessionId, const char *pVirtualName,
     if (wts->display_num < 0)
     {
         LOG(LOG_LEVEL_ERROR, "WTSVirtualChannelOpenEx: fatal error; invalid DISPLAY");
-        free(wts);
+        *errcode = WTS_E_RESOURCE_ERROR;
+        free_wts(wts);
         return NULL;
     }
 
@@ -124,7 +145,8 @@ WTSVirtualChannelOpenEx(unsigned int SessionId, const char *pVirtualName,
     if ((wts->fd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0)
     {
         LOG(LOG_LEVEL_ERROR, "WTSVirtualChannelOpenEx: socket failed");
-        free(wts);
+        *errcode = WTS_E_RESOURCE_ERROR;
+        free_wts(wts);
         return NULL;
     }
 
@@ -155,7 +177,8 @@ WTSVirtualChannelOpenEx(unsigned int SessionId, const char *pVirtualName,
         else
         {
             LOG(LOG_LEVEL_ERROR, "WTSVirtualChannelOpenEx: connect failed");
-            free(wts);
+            *errcode = WTS_E_CHANSRV_NOT_UP;
+            free_wts(wts);
             return NULL;
         }
     }
@@ -164,51 +187,21 @@ WTSVirtualChannelOpenEx(unsigned int SessionId, const char *pVirtualName,
     if (!can_send(wts->fd, 500, 1))
     {
         LOG(LOG_LEVEL_ERROR, "WTSVirtualChannelOpenEx: can_send failed");
-        free(wts);
+        *errcode = WTS_E_CHANSRV_NOT_UP;
+        free_wts(wts);
         return NULL;
     }
 
-    chan_name_bytes = strlen(pVirtualName);
-    bytes = 4 + 4 + 4 + chan_name_bytes + 4;
+    cd.connect_data.version = XRDPAPI_CONNECT_PDU_VERSION;
+    cd.connect_data.private_chan = private_chan;
+    cd.connect_data.flags = flags;
+    snprintf(cd.connect_data.name, sizeof(cd.connect_data.name), "%s", pVirtualName);
 
-    LOG_DEVEL(LOG_LEVEL_DEBUG,
-              "WTSVirtualChannelOpenEx: chan_name_bytes %d bytes %d pVirtualName %s",
-              chan_name_bytes, bytes, pVirtualName);
-
-    connect_data = (char *) calloc(bytes, 1);
-    if (connect_data == NULL)
-    {
-        LOG(LOG_LEVEL_ERROR, "WTSVirtualChannelOpenEx: calloc failed");
-        free(wts);
-        return NULL;
-    }
-
-    connect_data[0] = (bytes >> 0) & 0xFF;
-    connect_data[1] = (bytes >> 8) & 0xFF;
-    connect_data[2] = (bytes >> 16) & 0xFF;
-    connect_data[3] = (bytes >> 24) & 0xFF;
-
-    /* version here(4-7), just leave 0 */
-
-    connect_data[8] = (chan_name_bytes >> 0) & 0xFF;
-    connect_data[9] = (chan_name_bytes >> 8) & 0xFF;
-    connect_data[10] = (chan_name_bytes >> 16) & 0xFF;
-    connect_data[11] = (chan_name_bytes >> 24) & 0xFF;
-
-    memcpy(connect_data + 12, pVirtualName, chan_name_bytes);
-
-    connect_data[4 + 4 + 4 + chan_name_bytes + 0] = (flags >> 0) & 0xFF;
-    connect_data[4 + 4 + 4 + chan_name_bytes + 1] = (flags >> 8) & 0xFF;
-    connect_data[4 + 4 + 4 + chan_name_bytes + 2] = (flags >> 16) & 0xFF;
-    connect_data[4 + 4 + 4 + chan_name_bytes + 3] = (flags >> 24) & 0xFF;
-
-    LOG_DEVEL(LOG_LEVEL_DEBUG,
-              "WTSVirtualChannelOpenEx: calling mysend with %d bytes", bytes);
-
-    if (mysend(wts->fd, connect_data, bytes) != bytes)
+    if (mysend(wts->fd, &cd, sizeof(cd)) != sizeof(cd))
     {
         LOG(LOG_LEVEL_ERROR, "WTSVirtualChannelOpenEx: mysend failed");
-        free(wts);
+        *errcode = WTS_E_RESOURCE_ERROR;
+        free_wts(wts);
         return NULL;
     }
     LOG_DEVEL(LOG_LEVEL_DEBUG, "WTSVirtualChannelOpenEx: sent ok");
@@ -216,29 +209,75 @@ WTSVirtualChannelOpenEx(unsigned int SessionId, const char *pVirtualName,
     if (!can_recv(wts->fd, 500, 1))
     {
         LOG(LOG_LEVEL_ERROR, "WTSVirtualChannelOpenEx: can_recv failed");
-        free(wts);
+        *errcode = WTS_E_RESOURCE_ERROR;
+        free_wts(wts);
         return NULL;
     }
 
     /* get response */
-    if (myrecv(wts->fd, connect_data, 4) != 4)
+    if (myrecv(wts->fd, &connect_result, sizeof(connect_result)) !=
+            sizeof(connect_result))
     {
         LOG(LOG_LEVEL_ERROR, "WTSVirtualChannelOpenEx: myrecv failed");
-        free(wts);
+        *errcode = WTS_E_RESOURCE_ERROR;
+        free_wts(wts);
         return NULL;
     }
 
-    if ((connect_data[0] != 0) || (connect_data[1] != 0) ||
-            (connect_data[2] != 0) || (connect_data[3] != 0))
+    if (connect_result != 0)
     {
         LOG(LOG_LEVEL_ERROR, "WTSVirtualChannelOpenEx: connect_data not ok");
-        free(wts);
+        *errcode = WTS_E_RESOURCE_ERROR;
+        free_wts(wts);
         return NULL;
     }
 
     return wts;
 }
 
+/*
+ * Opens a handle to the server end of a specified virtual channel - this
+ * call is deprecated - use WTSVirtualChannelOpenEx() instead
+ *
+ * @param  hServer       - *must* be WTS_CURRENT_SERVER_HANDLE
+ * @param  SessionId     - current session ID; *must* be WTS_CURRENT_SESSION
+ * @param  pVirtualName  - virtual channel name when using SVC
+ *                       - name of endpoint listener when using DVC
+ *
+ * @return a valid pointer on success, NULL on error
+ ******************************************************************************/
+void *
+WTSVirtualChannelOpen(void *hServer, unsigned int SessionId,
+                      const char *pVirtualName)
+{
+    enum wts_errcode errcode_dummy = WTS_E_NO_ERROR;
+
+    if (hServer != WTS_CURRENT_SERVER_HANDLE)
+    {
+        return 0;
+    }
+    return VirtualChannelOpen(SessionId, pVirtualName, 0, 0, &errcode_dummy);
+}
+
+/*
+ * Opens a handle to the server end of a specified virtual channel
+ *
+ * @param  SessionId     - current session ID; *must* be WTS_CURRENT_SESSION
+ * @param  pVirtualName  - virtual channel name when using SVC
+ *                       - name of endpoint listener when using DVC
+ * @param  flags         - type of channel and channel priority if DVC
+ *
+ * @return a valid pointer on success, NULL on error
+ ******************************************************************************/
+void *
+WTSVirtualChannelOpenEx(unsigned int SessionId, const char *pVirtualName,
+                        unsigned int flags)
+{
+    enum wts_errcode errcode_dummy = WTS_E_NO_ERROR;
+    return VirtualChannelOpen(SessionId, pVirtualName,
+                              flags, 0, &errcode_dummy);
+}
+
 /*
  * Prevent receiving SIGPIPE on disconnect using either MSG_NOSIGNAL (Linux)
  * or SO_NOSIGPIPE (Mac OS X)
@@ -307,6 +346,25 @@ myrecv(int sck, void *adata, int bytes)
     return recd;
 }
 
+/*****************************************************************************/
+static int
+mypeek(int sck, void *adata, int bytes)
+{
+    int error;
+
+#if defined(SO_NOSIGPIPE)
+    const int on = 1;
+    setsockopt(sck, SOL_SOCKET, SO_NOSIGPIPE, &on, sizeof(on));
+#endif
+
+    error = 0;
+    if (can_recv(sck, 100, 0))
+    {
+        error = recv(sck, adata, bytes, MSG_NOSIGNAL | MSG_PEEK);
+    }
+    return error;
+}
+
 /*
  * write data to client connection
  *
@@ -406,21 +464,14 @@ WTSVirtualChannelRead(void *hChannelHandle, unsigned int TimeOut,
 int
 WTSVirtualChannelClose(void *hChannelHandle)
 {
-    struct wts_obj *wts;
-
-    wts = (struct wts_obj *)hChannelHandle;
+    struct wts_obj *wts = (struct wts_obj *)hChannelHandle;
 
     if (wts == NULL)
     {
         return 0;
     }
 
-    if (wts->fd != -1)
-    {
-        close(wts->fd);
-    }
-
-    free(wts);
+    free_wts(wts);
     return 1;
 }
 
@@ -535,3 +586,225 @@ can_recv(int sck, int millis, int restart)
 
     return rv;
 }
+
+/*****************************************************************************/
+int WTSQuerySessionInformationA(void *hServer,
+                                unsigned int SessionId,
+                                WTS_INFO_CLASS WTSInfoClass,
+                                void           *ppBuffer,
+                                DWORD          *pBytesReturned,
+                                enum wts_errcode *errcode)
+{
+    int rv = 0;
+    enum wts_errcode errcode_dummy;
+    if (errcode == NULL)
+    {
+        errcode = &errcode_dummy;
+    }
+    *errcode = WTS_E_NO_ERROR;
+
+    if (hServer != WTS_CURRENT_SERVER_HANDLE)
+    {
+        LOG(LOG_LEVEL_ERROR, "WTSQuerySessionInformationA: bad hServer");
+        *errcode = WTS_E_BAD_SERVER;
+    }
+    else if (SessionId != WTS_CURRENT_SESSION)
+    {
+        LOG(LOG_LEVEL_ERROR, "WTSQuerySessionInformationA: bad SessionId");
+        *errcode = WTS_E_BAD_SESSION_ID;
+    }
+    else if (WTSInfoClass != WTSConnectState)
+    {
+        LOG(LOG_LEVEL_ERROR,
+            "WTSQuerySessionInformationA: unsupported WTSInfoClass");
+        *errcode = WTS_E_BAD_INFO_CLASS;
+    }
+    else
+    {
+        rv = 1; // Assume success
+        if (wts_current_server.info_obj == NULL)
+        {
+            // We don't have a current connection for server state events.
+            // Set one up to update our cached values, then tear it down again.
+            int fd;
+
+            rv = WTSRegisterSessionNotificationEx(hServer, &fd, 0, errcode) &&
+                 WTSUnRegisterSessionNotificationEx(hServer, fd, errcode);
+        }
+
+        if (rv == 1)
+        {
+            *(WTS_CONNECTSTATE_CLASS *)ppBuffer =
+                (wts_current_server.session_state.is_connected)
+                ? WTSConnected
+                : WTSDisconnected;
+            if (pBytesReturned != NULL)
+            {
+                *pBytesReturned = sizeof(WTS_CONNECTSTATE_CLASS);
+            }
+        }
+    }
+
+    return rv;
+}
+
+/*****************************************************************************/
+int WTSRegisterSessionNotificationEx(void *hServer,
+                                     int *fd_ptr,
+                                     int dwFlags,
+                                     enum wts_errcode *errcode)
+{
+    (void)dwFlags;  // Unused parameter
+    enum wts_errcode errcode_dummy;
+    if (errcode == NULL)
+    {
+        errcode = &errcode_dummy;
+    }
+    *errcode = WTS_E_NO_ERROR;
+
+    if (hServer != WTS_CURRENT_SERVER_HANDLE)
+    {
+        LOG(LOG_LEVEL_ERROR, "WTSRegisterSessionNotificationEx: bad hServer");
+        *errcode = WTS_E_BAD_SERVER;
+        return 0;
+    }
+
+    if (wts_current_server.info_obj == NULL)
+    {
+        // Open a private channel for session info messages
+        // The name and flags args are ignored for xrdp private channels
+        struct wts_obj *wts = (struct wts_obj *)
+                              VirtualChannelOpen(WTS_CURRENT_SESSION,
+                                  "", 0,
+                                  CHAN_ID_XRDP_SESSION_INFO,
+                                  errcode);
+        if (wts != NULL)
+        {
+            // Server will pass the current session state now
+            struct xrdp_chan_session_state sess_state;
+            if (!can_recv(wts->fd, 1000, 1))
+            {
+                LOG(LOG_LEVEL_ERROR,
+                    "WTSRegisterSessionNotificationEx: can_recv failed");
+                *errcode = WTS_E_RESOURCE_ERROR;
+                free_wts(wts);
+            }
+            /* get server_status */
+            else if (myrecv(wts->fd, &sess_state, sizeof(sess_state)) !=
+                     sizeof(sess_state))
+            {
+                LOG(LOG_LEVEL_ERROR,
+                    "WTSRegisterSessionNotificationEx: myrecv failed");
+                *errcode = WTS_E_RESOURCE_ERROR;
+                free_wts(wts);
+            }
+            else
+            {
+                wts_current_server.info_obj = wts;
+                wts_current_server.session_state = sess_state;
+            }
+        }
+    }
+    if (wts_current_server.info_obj == NULL)
+    {
+        *fd_ptr = -1;
+        return 0;
+    }
+    *fd_ptr = wts_current_server.info_obj->fd;
+    return 1;
+}
+
+/*****************************************************************************/
+int WTSUnRegisterSessionNotificationEx(void *hServer,
+                                       int fd,
+                                       enum wts_errcode *errcode)
+{
+    (void)fd;  // Unused parameter
+    enum wts_errcode errcode_dummy;
+    if (errcode == NULL)
+    {
+        errcode = &errcode_dummy;
+    }
+    *errcode = WTS_E_NO_ERROR;
+
+    if (hServer != WTS_CURRENT_SERVER_HANDLE)
+    {
+        LOG(LOG_LEVEL_ERROR, "WTSUnRegisterSessionNotificationEx: bad hServer");
+        *errcode = WTS_E_BAD_SERVER;
+        return 0;
+    }
+
+    free_wts(wts_current_server.info_obj);
+    wts_current_server.info_obj = NULL;
+    return 1;
+}
+
+/*****************************************************************************/
+int
+WTSGetDispatchMessage(void *cbdata, WNDPROC wndproc, LRESULT *lResult)
+{
+    LRESULT result = 0;
+    struct xrdp_chan_session_state new_state;
+
+    /* get response */
+    if (wts_current_server.info_obj == NULL)
+    {
+        LOG(LOG_LEVEL_ERROR, "WTSGetDispatchMessage: No notification channel was opened");
+    }
+    else if (!can_recv(wts_current_server.info_obj->fd, 0, 1))
+    {
+        // No message available - nothing to log
+    }
+    else if (mypeek(wts_current_server.info_obj->fd,
+                    &new_state, sizeof(new_state)) != sizeof(new_state))
+    {
+        LOG(LOG_LEVEL_ERROR, "WTSGetDispatchMessage: An incomplete message was received");
+    }
+    else
+    {
+        /* We've peeked a message. Find a SINGLE difference with our
+         * own state, update our state, and issue a callback. If there
+         * are more differences, the application will call us back and
+         * we can process the next one. When all the differences are
+         * accounted for, we can clear the message from the queue, and
+         * the fd will no longer be readable */
+        UINT msgno = 0; // 0 means 'no message'
+        WPARAM wParam = 0;
+        LPARAM lParam = 0;
+        // Look for a single difference in the state we have, and the
+        // current state from the server
+        struct xrdp_chan_session_state *curr_state;
+        curr_state = &wts_current_server.session_state;
+
+        if (curr_state->is_connected != new_state.is_connected)
+        {
+            curr_state->is_connected = new_state.is_connected;
+            msgno = WM_WTSSESSION_CHANGE;
+            wParam = (new_state.is_connected)
+                     ? WTS_REMOTE_CONNECT : WTS_REMOTE_DISCONNECT;
+        }
+
+        // If we found a difference, activate the callback
+        if (msgno != 0)
+        {
+            *lResult = wndproc(cbdata, msgno, wParam, lParam);
+            result = 1;
+        }
+
+        // If we've exhausted all the differences between the old and
+        // the new state, purge the message from the queue
+        if (curr_state->is_connected == new_state.is_connected &&
+                /* Add further checks here in the future */
+                1)
+        {
+            /* Sanity check on the fd */
+            if (wts_current_server.info_obj->fd >= 0)
+            {
+                (void)myrecv(wts_current_server.info_obj->fd,
+                             &new_state, sizeof(new_state));
+            }
+        }
+    }
+
+    return result;
+}
